/* REXX
  
   Prints info on all smb responding machines on a subnet.
   This script needs to be run on a machine without nmbd running and be
   run as root to get correct info from WIN95 clients.
  
   syntax:
      findsmb [-d|-D] [-r] [subnet broadcast address]
  
   with no agrument it will list machines on the current subnet
  
   There will be a "+" in front of the workgroup name for machines that are
   local master browsers for that workgroup. There will be an "*" in front
   of the workgroup name for machines that are the domain master browser for
   that workgroup.
   
   Options:
  
   -d|-D		enable debug
   -r		add -r option to nmblookup when finding netbios name
 */

'@echo off'
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs
        
parse source . . findsmb
samba.!bin = filespec('D',findsmb)||strip(filespec('P',findsmb),'T','\')

parse arg OrgCmdLine
CmdLine = translate(OrgCmdLine)

R_Option = ""
Debug = 0
BCast = ""
Master= " "
smbconferror = 0
ExtMode = 0
defaultworkgroup =""

nmblookup = value("TMPDIR",,"OS2ENVIRONMENT")'\nmblookup.txt'
smbtree   = value("TMPDIR",,"OS2ENVIRONMENT")'\smbtree.txt'
smb       = value("TMPDIR",,"OS2ENVIRONMENT")'\smb.txt'

ok = SysFileDelete(nmblookup)
ok = SysFileDelete(smbtree)
ok = SysFileDelete(smb)

/* test for -d and -r options and
   set broadcast address if it was specified */

do i = 1 to words(Cmdline)
    if word(CmdLine,i) = '-R'| word(CmdLine,i) = '/R' then R_Option = "-r"
    if word(CmdLine,i) = '-D'| word(CmdLine,i) = '/D' then Debug    = 1
    if word(CmdLine,i) = '-B'| word(CmdLine,i) = '/B' then BCast    = "-B "word(CmdLine,I+1) /* error check missing here */
    if word(CmdLine,i) = '-X'| word(CmdLine,i) = '/X' then Extmode  = 1
end

if ExtMode then do
    call charout , 'Please stand by during extended data collection.'
    samba.!bin"\smbtree --debuglevel=0 -N 1>"smbtree    
    do until lines(smbtree) = 0
        smbtreeline = linein(smbtree)
        if left(smbtreeline,4)='tdb(' then iterate
        if defaultworkgroup = "" then defaultworkgroup = smbtreeline
        if left(smbtreeline,2) = '09'x||'\' then do
            smbtreeline=substr(smbtreeline,1,)
            if Debug then say smbtreeline
            parse var smbtreeline '\\' host .
            call charout , '.'
            samba.!bin"\nmblookup "BCAST" '"host"' --debuglevel=0 >>"nmblookup
        end
    end
    call charout , '0D'x||copies(" ",79)||'0D'x
end
else do /* look for all machines that respond to a name lookup */
    samba.!bin"\nmblookup "BCAST" '*' --debuglevel=0 >"nmblookup
end

/* get rid of all lines that are not a response IP address,
strip everything but IP address and sort by last field in address */

I = 0
LastIP = ''
do until lines(nmblookup) = 0
    nmbline = linein(nmblookup)
    if pos("<00>",nmbline) > 0 then do
        I = I + 1
        ipaddrs.I = word(nmbline,1)
        if ipaddrs.I <> LastIP then do
            If I > 0 then LastIP = ipaddrs.I
            parse var ipaddrs.I part.1'.'part.2'.'part.3'.'part.4
            do J = 1 to 4
                part.J = right('000'part.J,3)
            end
            ipaddrs.I = part.1'.'part.2'.'part.3'.'part.4
        end
        else I = I - 1
    end
end
ok = stream(nmblookup,'c','close')
ipaddrs.0 = I /* stem is not sorted yet */

if ipaddrs.0 > 0 then do
    if RxFuncQuery("SysStemSort") = 0 then do
        ok = SysStemSort("ipaddrs","A","I")
    end
    else say "Warning! Your RexxUtil.DLL is outdated, please update!"
end
else say "No machines responded on this subnet!"

/* strip leading 0 from addresses */
do I = 1 to ipaddrs.0
    parse var ipaddrs.I part.1'.'part.2'.'part.3'.'part.4
    do J = 1 to 4
        part.J = strip(part.J,'L','0')
    end
    ipaddrs.I = part.1'.'part.2'.'part.3'.'part.4
end

/* print header info */
say "                                *=DMB"
say "                                +=LMB"
say "IP ADDR         NETBIOS NAME     WORKGROUP/OS/VERSION "BCast
say copies("-",79)

do I = 1 to ipaddrs.0 /* loop through each IP address found */
    Name      = "unknown"
    Master    = " "
    Workgroup = "unknown"
    Server    = "unknown"
    OS        = "unknown"
    IsWin95   = 0

	/* find the netbios names registered by each machine */
    samba.!bin"\nmblookup "R_Option" -A "ipaddrs.I" --debuglevel=0 >"nmblookup
    do until lines(nmblookup) = 0
        nmbline = linein(nmblookup)
        if pos('<1b>', nmbline) > 0 then Master = "+"
        if pos('<1d>', nmbline) > 0 then Master = "*"
        if pos('<00>', nmbline) > 0 then do /* we have a netbios name */
            if pos('GROUP',nmbline) > 0 then do /* is it a group? */
                WorkGroup = strip(left(strip(nmbline,,'09'x),15))
            end
            else do
                /* The Netbios name can contain lot of characters also < >
                   and spaces. The follwing cure inside name space but not
                   names starting or ending with spaces */
                name = strip(left(strip(nmbline,,'09'x),15))
                samba.!bin"\smbclient -L "name" -I "ipaddrs.I" -N --debuglevel=1 2>"smb" 1>NUL"
                smbline = linein(smb)
                /* say smbline */
                smbconferror = 1
                if left(translate(smbline),7) = "UNKNOWN" then smbline = linein(smb)
                                                          else smbconferror = 0
                if left(translate(smbline),8) = "IGNORING" then smbline = linein(smb)
                                                           else smbconferror = 0
                ok = stream(smb,'c','close') 
                if Debug then do /* Debug */
                    say copies("=",79)
                    'type 'nmblookup
                    'type 'smb
                    say copies("=",79)
                end
                if pos('OS=[',smbline) = 0 then do /* WIN95 client */
                    /* not properly supported yet */
                    OS="Win 95"
                    IsWin95 = 1
                end
                else do
                    if Debug then say "smbline "smbline
                    parse var smbline "Domain=["WorkGroup"] OS=["OS"] Server=["Server"]"Rest
                    if pos("OS/2",OS) > 0 then OS = "OS/2 (eCS)"
                    if word(OS,1) = "Windows" then OS = "Win"substr(OS,8)
                    if word(Server,1) = "Windows" then Server = "Win"substr(Server,8)
                end
            end
        end
    end
    ok = stream(nmblookup,'c','close')
    say left(left(ipaddrs.I,16)||left(Name,15)||left(Master,2)||WorkGroup'/'OS'/'Server,79)
end

ok = SysFileDelete(nmblookup)
ok = SysFileDelete(smb)
if smbconferror then do
    say
    say 'Warning! Errors in smb.conf detected! Run testparm!'
end
exit
